program InternetConfig;

(*	This file is part of the Internet Configuration system and is placed in the public domain for the benefit of all.

	This module implements the main entry point of the Internet Config application.
	
	IC is complicated by the fact that it must run under both System 6 and System 7.
	This means that we have to deal with both AppleEvent and AppParameters. These
	differences are entirely contained within this file.
	
	The rest of the file is made up of code that simply initialises the rest of the program.

	Useful Global Knowledge
	----------------------
	
	1. The source consistently uses certain prefixes to identify certain classes of
		identifiers. These include:

			kXxx			-- constant
			gXxx			-- global variable (used even for globals internal to individual modules)
			mXxx		-- menu ID
			iXxx			-- menu item number
			rXxx			-- resource ID
			strXxx		-- index into a STR# resource
			ditXxx		-- dialog item number
			WT_Xxx	-- Window Type, see ICWindowGlobals for a description
			acXxx		-- Error actions, see ICGlobals
			
		Also, the source follows the usual Macintosh naming conventions for routines
		(in MixedCase) and parameters/locals (withLowerCaseFirstLetter).
	
	2. Every now and again I turn on Case Sensitive Identifiers in Metrowerks
		Pascal to root out all the places where I've used the wrong case.
		
	3. mention memory management issues, specifically the fact that
		I don't worry about NewUPP failing because all of them are done very
		early.
*)

	uses
		SegLoad,
		Fonts,
		TextEdit,
		Dialogs,
		GestaltEqu,
		Traps,
		
		ICText,
		ICDialogs,
		ICStandardFile,
		ICResources,
		ICGlobals, 
		ICDocument,
		ICEvents, 
		ICWindows, 
		ICInstall, 
		ICIconSuites, 
		ICCommonSubs,
		ICMiscSubs, 
		ICMiscAppleEvents;

	function HandleAEOpenApplication (var theAppleEvent : AppleEvent; var reply: AppleEvent; refcon : longint): OSErr;
		(* Handle the Open Application AppleEvent. Basically this does the standard AppleEvent stuff, then passes
			the event off to the DoOpenDefaultDocument routine in ICDocument.
		*)
		var
			err: OSStatus;
	begin
		{$unused reply}
		{$unused refcon}
		err := AEGotRequiredParams(theAppleEvent);
		if err = noErr then begin
			err := DoOpenDefaultDocument;
			if err <> noErr then begin
				if AEInteractWithUser(kAEDefaultTimeout, nil, nil) = noErr then begin
					DisplayError(acNewDocument, err);
				end; (* if *)
			end; (* if *)
		end; (* if *)
		HandleAEOpenApplication := err;
	end; (* HandleAEOpenApplication *)

	function HandleAEOpenDocuments (var theAppleEvent : AppleEvent; var reply: AppleEvent; refcon : longint): OSErr;
		(* Handle the Open Documents AppleEvent. Basically this does the standard AppleEvent stuff, then passes
			the event off to the DoOpenDocument routine in ICDocument.
		*)
		var
			err : OSStatus;
			docList: AEDescList;
			numberOfItemsInList: longint;
			index : longint;
			docToOpen: FSSpec;
			junkKeyword: AEKeyword;
			junkActualSize: Size;
			junkTypeCode: DescType;
			junkErr : OSStatus;
	begin
		{$unused reply}
		{$unused refcon}
		err := AEGetParamDesc(theAppleEvent, keyDirectObject, typeAEList, docList);
		if err = noErr then begin
			err := AEGotRequiredParams(theAppleEvent);
			if err = noErr then begin
				err := AECountItems(docList, numberOfItemsInList);
			end; (* if *)
			if err = noErr then begin
				for index := 1 to numberOfItemsInList do begin
					if AEGetNthPtr(docList, index, typeFSS, junkKeyword, junkTypeCode, @docToOpen, sizeof(docToOpen), junkActualSize) = noErr then begin
						err := DoOpenDocument(docToOpen);
						if err = noErr then begin
							if AEInteractWithUser(kAEDefaultTimeout, nil, nil) = noErr then begin
								DisplayError(acOpenDocument, err);
							end; (* if *)
						end; (* if *)
					end; (* if *)
				end; (* for *)
				err := noErr;
			end; (* if *)
			junkErr := AEDisposeDesc(docList);
		end; (* if *)
		HandleAEOpenDocuments := err;
	end; (* HandleAEOpenDocuments *)

	function HandleAEQuitApplication (var theAppleEvent, reply: AppleEvent; refcon : longint): OSErr;
		(* Handle the Quit AppleEvent. Basically this does the standard AppleEvent stuff, then calls
			the DoCloseDocument routine in ICDocuments. If this returns noErr, we set the gQuitNow Boolean
			which causes the application to exit the main event loop.
		*)
		var
			err: OSStatus;
	begin
		{$unused reply}
		{$unused refcon}
		err := AEGotRequiredParams(theAppleEvent);
		if err = noErr then begin
			err := DoQuit;
		end; (* if *)
		HandleAEQuitApplication := err;
	end; (* HandleAEQuitApplication *)

	procedure HandleAppParams;
		(* This routine is called at application startup time. It's responsible for either registering the
			AppleEvent handlers in preparation for the main event loop getting the initial AppleEvents,
			or for reading through the AppParameters list and doing the right thing.
			Well it used to do both.  Now it just has System 7 code.
		*)
		var
			junk: OSStatus;
	begin
		(* Do it the System 7 way. *)
		junk := AEInstallEventHandler(kCoreEventClass, kAEOpenApplication,
								NewAEEventHandlerProc(HandleAEOpenApplication), 0, false);
		junk := AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments, 
								NewAEEventHandlerProc(HandleAEOpenDocuments), 0, false);
		junk := AEInstallEventHandler(kCoreEventClass, kAEQuitApplication, 
								NewAEEventHandlerProc(HandleAEQuitApplication), 0, false);
		InstallMiscAppleEventHandlers;
	end; (* HandleAppParams *)

{$IFC not GENERATINGPOWERPC}
	function Get68KStackPointerRegister: longint;
	inline
		$2E8F;				(* 	move.l a7,(sp) 	*)
{$ENDC}

	procedure Initialization;
		(* The canonical Toolbox initialisation stuff. Why is this canonical?  Because this is the sequence that
			Think Pascal would execute unless you turn off the auto-initialisation option.
		*)
		var
			i: integer;
	begin
		InitGraf(@qd.thePort);
		InitFonts;
		InitWindows;
		InitMenus;
		TEInit;
		InitDialogs(nil);
{$IFC not GENERATINGPOWERPC}
		(* Guarantee that we have at least 32K of stack on 68K machines. *)
		SetApplLimit(Ptr(Get68KStackPointerRegister - 32768));
{$ENDC}
		MaxApplZone;
		for i := 1 to 3 do begin
			MoreMasters;
		end; (* for *)
	end; (* Initialization *)
	
	procedure DieYoung(alertID : integer);
		var
			junk : integer;
	begin
		InitCursor;
		junk := StopAlert(alertID, nil);
		ExitToShell;
	end; (* DieYoung *)

	var
		mainMenuBar: Handle;
		err: OSStatus;
		gestaltValue : longint;
begin
	Initialization;
	
	(* Some clown is trying to run us on a system prior to System 7.  We're
		not prepare to display an error message via the normal means this early
		in the init sequence.  So instead, we use a special case error dialog.
	*)
	if not TrapAvailable(_Gestalt) | 
			(Gestalt(gestaltSystemVersion, gestaltValue) <> noErr) | 
			(gestaltValue < $0700) then begin
		DieYoung(rNeedSystem7Alert);
	end; (* if *)
	
	(*	Check for the Component Manager, and bail now, without trying
		anything else.
	*)
	if (Gestalt(gestaltComponentMgr, gestaltValue) <> noErr) | (gestaltValue = 0) then begin
		DieYoung(rNeedSystemComponentManagerAlert);
	end; (* if *)
	
	(* Set up the menu bar. *)
	mainMenuBar := GetNewMBar(rMainMenuBar);
	if mainMenuBar = nil then begin
		ExitToShell;		(* We're still not prepared to handle errors yet. *)
	end; (* if *)
	SetMenuBar(mainMenuBar);
	AppendResMenu(GetMenuHandle(mApple), 'DRVR');
	DrawMenuBar;
	
	(* Initialise some modules that the rest of the code depends on. *)
	InitICText;
	InitICGlobals;
	InitListManagerMiscSubs;
	InitICIconSuites;
	InitICDialogs;
	InitICStandardFile;
	
	(* By this stage, we're prepared to Handle failure. *)
	err := InitICWindowGlobals;
	if err = noErr then begin
		err := InitICWindows;
	end; (* if *)
	if err = noErr then begin
		(* Install or update the component if necessary. *)
		err := InitializeComponentInstallation;
	end; (* if *)
	if err = noErr then begin
		err := InitICDocument;		(* This opens the primary connection to IC. *)
	end; (* if *)
	if err <> noErr then begin
		(* Handle an error during startup by displaying and then quitting. *)
		DisplayError(acStartApplication, err);
		ExitToShell;
	end; (* if *)
	
	(* We're now fully initialised and prepared to run the application proper. First check
		for any documents to open, then start the main event loop.
	*)
	HandleAppParams;
	InitCursor;
	while not gQuitNow do begin
		HandleAnEvent;
	end; (* while *)
	
	TermICInstall;					(* This closes the primary connection to IC. *)
end. (* InternetConfig *)